@echo off
setlocal EnableDelayedExpansion

rem Get the output of 'help' command and define a two-level menu
rem that can be displayed and navigated via CmdHelp.bat program
rem Antonio Perez Ayala

rem Create the help file for CmdHelp.bat program
cd "%~P0"
(
echo/
echo Help on CmdHelp.bat: How navigate the menu via keyboard and mouse.
echo/
echo Menu navigation                Key                 Mouse click
echo/
echo - Open this help screen        F1                  On blue "?" at top left
echo - Open a menu option           Digit or letter     On the option line
echo - Back to previous page        Enter               On white "<--" at top left
echo - Close menu                   Alt-F4              On red "x" at top right
echo/
echo When browsing a command description you may use the standard methods to
echo navigate the screen: a mouse click on the vertical scroll bar or using the
echo Home, PageUp, UpArrow, DownArrow, PageDown and End standard navigation keys.
echo/
) > CmdHelpF1.txt

rem Define the sets of commands for menu options
set "all=;"
set "nType=0"
for %%a in (
        "SCREEN:Commands for Screen control"
               "TITLE;COLOR;CLS;MODE;CHCP"
          "FILE:Commands to manage Files"
               "DIR;TREE;REN;RENAME;COPY;XCOPY;ROBOCOPY;MOVE;REPLACE;DEL;ERASE;ATTRIB;CACLS;ICACLS;MAKECAB;EXPAND;PACKAGER;MKLINK"
          "TEXT:Commands to process Text files"
               "TYPE;MORE;PRINT;FIND;FINDSTR;SORT;FC;COMP;EDLIN;CLIP"
     "DIRECTORY:Commands to create, change and remove Directories"
               "MD;MKDIR;CD;CHDIR;PUSHD;POPD;RD;RMDIR;SUBST;APPEND;MOUNTVOL"
          "DISK:Commands for Disk maintenance"
               "FORMAT;LABEL;VOL;CHKDSK;CHKNTFS;CONVERT;COMPACT;CIPHER;RECOVER;DISKCOPY;DISKCOMP;FSUTIL"
       "PROCESS:Commands to execute, view or cancel Processes"
               "START;RUNAS;AT;SCHTASKS;TASKLIST;TASKKILL;TSKILL;TSSHUTDN;SHUTDOWN;PATH;ASSOC;FTYPE;SETVER"
"COMMUNICATIONS:Commands to manage Communications"
               "MSG;TELNET;TFTP;PING;PATHPING;NET;OPENFILES"
         "BATCH:Commands used to write Batch files"
               "REM;ECHO;PAUSE;SHIFT;GOTO;IF;FOR;SET;CALL;CMD;EXIT;SETLOCAL;ENDLOCAL;Tables"
       "VARIOUS:Commands that not fits in previous divisions"
               "HELP;DATE;TIME;PROMPT;DOSKEY;VER;SYSTEMINFO"
   ) do for /F "tokens=1,2 delims=:" %%b in (%%a) do (
   if "%%c" neq "" (
      set /A nType+=1
      set "type[!nType!]=%%b"
      set "desc[!nType!]=%%c"
   ) else (
      set "list[!nType!]=;%%b;"
      set "all=!all!%%b;"
   )
)

rem Process the output of 'help' command and get the corresponding data
REM The "skip=2" option eliminate the first two lines from 'help' command
for /F "skip=2 delims=" %%a in ('help') do (
   set "line=%%a"
   if "!line:~0,1!" neq " " for /F %%b in ("%%a") do (
      set "desc[%%b]=%%a"
      if "!all:;%%b;=!" equ "%all%" set "list[%nType%]=!list[%nType%]!%%b;"
   )
)
REM The code below eliminate the last two lines from 'help' command
set "one=" & set "two=" & set "all=;"
set "list[%nType%]=!list[%nType%]:~1,-1!"
for %%a in ("!list[%nType%]:;=" "!") do (
   if defined one set "all=!all!!one!;"
   set "one=!two!"
   set "two=%%~a"
)
rem Eliminate unwanted/dangerous commands
for %%a in (DISKPART GRAFTABL SC) do set "all=!all:%%a;=!"
set "list[%nType%]=!all!"
set "all="
rem Special commands whose help pauses, so they requires > CON
set "special=;DIR;IF;FOR;SET;CALL;START;WMIC;"
rem Insert an additional Batch menu option
set "desc[Tables]=Tables         SET /A operators, parameter modifiers and dynamic variables."

rem Create the file with the data of all options in the menu
echo Creating menu
set "letter= ABCDEFGHIJKLMNOPQRSTUVWXYZ"
set "SP=                  " & REM 18 spaces to align the command
set "oneK=" & set "oneM="
set "last0=%nType%"
(
for /L %%i in (1,1,%nType%) do (
   echo %%i. !type[%%i]! > CON
   for /L %%j in (1,1,13) do if "!type[%%i]:~%%j!" neq "" set "j=%%j"
   for %%j in (!j!) do echo head[0%%i]=    %%i. !type[%%i]!!SP:~%%j!!desc[%%i]!
   set "oneK=!oneK!%%i"
   set /A i=%%i+4
   set "oneM=!oneM!  0 !i! 79 !i!"
   set "twoK=" & set "twoM=" & set /A "j=0, last1=0"
   for %%a in (!list[%%i]!) do (
      if not defined desc[%%a] for /F "delims=" %%b in ('%%a /? 2^>NUL') do (
         if not defined desc[%%a] (
            set "one=%%a%SP%" & set "two=%%b"
            REM 64 characters to cut longer lines
            set "desc[%%a]=!one:~0,15!!two:~0,64!"
         )
      )
      if defined desc[%%a] (
         set /A j+=1, i=j+7, last1+=1
         for %%j in (!j!) do (
            echo head[0%%i%%j]=!letter:~%%j,1!. !desc[%%a]:%%a   =%%a!
            set "twoK=!twoK!!letter:~%%j,1!"
         )
         set "twoM=!twoM!  0 !i! 79 !i!"
         set "help=%%a /?"
         if %%a neq Tables (
            if "!special:;%%a;=!" neq "%special%" ( set "help=%%a /? >CON"
            ) else if %%a equ CMD set "help=CMD /? | find /V "" "
         ) else ( rem "Tables" additional Batch menu option
            set "help=echo Operators for SET /A command&echo/&set /?|findstr /R /C:"^^     *[^^a-z %%^)]"&echo/&"
            set "help=!help!echo Modifiers for Batch file parameters&echo/&call /?|findstr /R /C:"^^^^ *%%~"&echo/&"
            set "help=!help!echo Modifiers for FOR command replaceable parameters&echo/&for /?|findstr /R /C:"^^^^ *%%~"&echo/&"
            set "help=!help!echo Dynamic variables&echo/&set /?|findstr "^^^^%%" "
         )
         echo help[0%%i!j!]=!help!
         call :makeHelp %%a
         for /F %%n in ('find /C /V "" ^< help.tmp') do set /A "last=%%n+14"
         set "last[0%%i!j!]=!last!"
      )
   )
   echo keys[0%%i]=!twoK!
   echo mous[0%%i]=!twoM!
   if !last1! gtr !last0! set "last0=!last1!"
)
echo keys[0]=!oneK!
echo mous[0]=!oneM!
set /A last0+=10
for /F "tokens=1,2 delims==" %%a in ('set last[') do (
   if %%b gtr !last0! (
      echo %%a=%%b
   ) else (
      echo %%a=!last0!
   )
)
for /L %%i in (1,1,%nType%) do echo last[0%%i]=!last0!
echo last[0]=!last0!
) > CmdHelp.txt
del help.tmp
endlocal
CmdHelp.bat
echo/
echo CmdHelp.bat file not found
goto :EOF

:makeHelp com
if %1 neq Tables (
   %help% > help.tmp
) else (
   ( %help% ) > help.tmp
)
exit /B
